#include "Field.hpp"
#include <MultiRegions/ExpList.h>

using namespace Nektar;
using namespace LibUtilities;

std::vector<BlockAttributes> GetBlockAttributes(
    FieldState state, const MultiRegions::ExpListSharedPtr explist,
    size_t VectorWidth)
{
    std::vector<BlockAttributes> blockAttr;

    // initialize the basisKeys
    std::vector<BasisKey> prevbasisKeys(3, NullBasisKey);
    std::vector<BasisKey> thisbasisKeys(3, NullBasisKey);
    int prevIsDeformed = -1, thisIsDeformed = -1;

    // initialize the first block using the first element
    auto expPtr = explist->GetExp(0);
    for (int d = 0; d < expPtr->GetNumBases(); d++)
    {
        prevbasisKeys[d] = expPtr->GetBasis(d)->GetBasisKey();
    }
    prevIsDeformed = expPtr->GetMetricInfo()->GetGtype();
    size_t num_pts = state == FieldState::Phys ? expPtr->GetTotPoints()
                                               : expPtr->GetNcoeffs();
    blockAttr.push_back({1, num_pts});

    // loop over elements
    for (int i = 1; i < explist->GetNumElmts(); i++)
    {
        expPtr = explist->GetExp(i);

        // fetch basiskeys of current element
        for (int d = 0; d < expPtr->GetNumBases(); d++)
        {
            thisbasisKeys[d] = expPtr->GetBasis(d)->GetBasisKey();
        }

        thisIsDeformed = expPtr->GetMetricInfo()->GetGtype();

        // if the basis is the same as the previous one,
        // increment the number of elements
        if (thisbasisKeys == prevbasisKeys && thisIsDeformed == prevIsDeformed)
        {
            blockAttr.back().num_elements++;
            blockAttr.back().block_size += blockAttr.back().num_pts;
        }
        else // if not, create a new block with the number of elements = 1
        {
            // compute the padding elements before creating a new block
            if (blockAttr.back().num_elements % VectorWidth == 0)
            {
                blockAttr.back().num_padding_elements = 0;
            }
            else
            {
                blockAttr.back().num_padding_elements =
                    VectorWidth - blockAttr.back().num_elements % VectorWidth;
                blockAttr.back().block_size +=
                    blockAttr.back().num_padding_elements *
                    blockAttr.back().num_pts;
            }
            // update num_pts for a new block
            num_pts = state == FieldState::Phys ? expPtr->GetTotPoints()
                                                : expPtr->GetNcoeffs();
            blockAttr.push_back({1, num_pts});
            prevbasisKeys  = thisbasisKeys;
            prevIsDeformed = thisIsDeformed;
        }
    }

    // update the padding elements for the last block
    // compute the padding elements
    if (blockAttr.back().num_elements % VectorWidth == 0)
    {
        blockAttr.back().num_padding_elements = 0;
    }
    else
    {
        blockAttr.back().num_padding_elements =
            VectorWidth - blockAttr.back().num_elements % VectorWidth;
        blockAttr.back().block_size +=
            blockAttr.back().num_padding_elements * blockAttr.back().num_pts;
    }

    return blockAttr;
}
